@@ -7,16 +7,7 @@
  *
  */
 #include "motion.h"
-
-#ifdef __freebsd__
-#include "video_freebsd.h"
-#else
-#include "video.h"
-#endif /* __freebsd__ */
-
-#include "conf.h"
 #include "alg.h"
-#include "event.h"
 
 #ifdef __MMX__
 #define HAVE_MMX
@@ -27,28 +18,27 @@
 #define MAX3(x, y, z) ((x) > (y) ? ((x) > (z) ? (x) : (z)) : ((y) > (z) ? (y) : (z)))
 
 /* locate the center and size of the movement. */
-struct coord alg_locate_center_size(struct images *imgs, int width, int height)
+void alg_locate_center_size(struct images *imgs, int width, int height, struct coord *cent)
 {
 	unsigned char *out=imgs->out;
 	int *labels=imgs->labels;
 	int x, y, centc=0, xdist=0, ydist=0;
-	struct coord cent;
 
-	cent.x=0;
-	cent.y=0;
-	cent.maxx=0;
-	cent.maxy=0;
-	cent.minx=width;
-	cent.miny=height;
-	
+	cent->x=0;
+	cent->y=0;
+	cent->maxx=0;
+	cent->maxy=0;
+	cent->minx=width;
+	cent->miny=height;
+
 	/* If Labeling enabled - locate center of largest labelgroup */
 	if (imgs->labelsize_max) {
 		/* Locate largest labelgroup */
 		for (y=0; y<height; y++) {
 			for (x=0; x<width; x++) {
 				if (*(labels++)&32768) {
-					cent.x += x;
-					cent.y += y;
+					cent->x += x;
+					cent->y += y;
 					centc++;
 				}
 			}
@@ -58,16 +48,16 @@ struct coord alg_locate_center_size(struct images *imgs, int width, int height)
 		for (y=0; y<height; y++) {
 			for (x=0; x<width; x++) {
 				if (*(out++)) {
-					cent.x += x;
-					cent.y += y;
+					cent->x += x;
+					cent->y += y;
 					centc++;
 				}
 			}
 		}
 	}
 	if (centc) {
-		cent.x=cent.x/centc;
-		cent.y=cent.y/centc;
+		cent->x=cent->x/centc;
+		cent->y=cent->y/centc;
 	}
 	
 	/* Now we find the size of the Motion */
@@ -82,14 +72,14 @@ struct coord alg_locate_center_size(struct images *imgs, int width, int height)
 		for (y=0; y<height; y++) {
 			for (x=0; x<width; x++) {
 				if (*(labels++)&32768) {
-					if (x > cent.x)
-						xdist += x - cent.x;
-					else if (x < cent.x)
-						xdist += cent.x - x;
-					if (y > cent.y)
-						ydist += y - cent.y;
-					else if (y < cent.y)
-						ydist += cent.y - y;
+					if (x > cent->x)
+						xdist += x - cent->x;
+					else if (x < cent->x)
+						xdist += cent->x - x;
+					if (y > cent->y)
+						ydist += y - cent->y;
+					else if (y < cent->y)
+						ydist += cent->y - y;
 					centc++;
 				}
 			}	
@@ -98,14 +88,14 @@ struct coord alg_locate_center_size(struct images *imgs, int width, int height)
 		for (y=0; y<height; y++) {
 			for (x=0; x<width; x++) {
 				if (*(out++)) {
-					if (x > cent.x)
-						xdist += x - cent.x;
-					else if (x < cent.x)
-						xdist += cent.x - x;
-					if (y > cent.y)
-						ydist += y - cent.y;
-					else if (y < cent.y)
-						ydist += cent.y - y;
+					if (x > cent->x)
+						xdist += x - cent->x;
+					else if (x < cent->x)
+						xdist += cent->x - x;
+					if (y > cent->y)
+						ydist += y - cent->y;
+					else if (y < cent->y)
+						ydist += cent->y - y;
 					centc++;
 				}
 			}	
@@ -113,51 +103,50 @@ struct coord alg_locate_center_size(struct images *imgs, int width, int height)
 	}
 	
 	if (centc) {
-		cent.minx = cent.x - xdist/centc*2;
-		cent.maxx = cent.x + xdist/centc*2;
-		/* Make the box a litle bigger in y direction to make sure the
+		cent->minx = cent->x - xdist/centc*2;
+		cent->maxx = cent->x + xdist/centc*2;
+		/* Make the box a little bigger in y direction to make sure the
 		   heads fit in so we multiply by 3 instead of 2 which seems to
 		   to work well in practical */
-		cent.miny = cent.y - ydist/centc*3;
-		cent.maxy = cent.y + ydist/centc*2;
+		cent->miny = cent->y - ydist/centc*3;
+		cent->maxy = cent->y + ydist/centc*2;
 	}
-	if (cent.maxx > width - 1)
-		cent.maxx = width - 1;
-	else if (cent.maxx < 0)
-		cent.maxx = 0;
-	if (cent.maxy > height - 1)
-		cent.maxy = height - 1;
-	else if (cent.maxy < 0)
-		cent.maxy = 0;
-	if (cent.minx > width - 1)
-		cent.minx = width - 1;
-	else if (cent.minx < 0)
-		cent.minx = 0;
-	if (cent.miny > height - 1)
-		cent.miny = height - 1;
-	else if (cent.miny < 0)
-		cent.miny = 0;
+	if (cent->maxx > width - 1)
+		cent->maxx = width - 1;
+	else if (cent->maxx < 0)
+		cent->maxx = 0;
+	if (cent->maxy > height - 1)
+		cent->maxy = height - 1;
+	else if (cent->maxy < 0)
+		cent->maxy = 0;
+	if (cent->minx > width - 1)
+		cent->minx = width - 1;
+	else if (cent->minx < 0)
+		cent->minx = 0;
+	if (cent->miny > height - 1)
+		cent->miny = height - 1;
+	else if (cent->miny < 0)
+		cent->miny = 0;
 	
-	cent.width = cent.maxx - cent.minx;
-	cent.height = cent.maxy - cent.miny;
+	cent->width = cent->maxx - cent->minx;
+	cent->height = cent->maxy - cent->miny;
 	
 	/* We want to center Y coordinate to be the center of the action.
 	   The head of a person is important so we correct the cent.y coordinate
 	   to match the correction to include a persons head that we just did above */
-	cent.y = (cent.miny + cent.maxy)/2;
+	cent->y = (cent->miny + cent->maxy)/2;
 	
-	return (cent);
 }
 
 
 /* draw a box around the movement */
-void alg_draw_location(struct coord *cent, struct images *imgs, int width, int height, unsigned char *new, int mode)
+void alg_draw_location(struct coord *cent, struct images *imgs, int width, unsigned char *new, int mode)
 {
 	unsigned char *out=imgs->out;
 	int x, y;
 
 	out=imgs->out;
-	
+
 	/* Draw a box around the movement */
 	if (mode == LOCATE_BOTH){ /* both normal and motion image gets a box */
 		int width_miny = width*cent->miny;
@@ -205,7 +194,7 @@ void alg_draw_location(struct coord *cent, struct images *imgs, int width, int h
 #define NDIFF(x, y)        (ABS(x)*NORM/(ABS(x)+2*DIFF(x,y)))
 
 
-void alg_noise_tune (struct context *cnt, unsigned char *new)
+void alg_noise_tune(struct context *cnt, unsigned char *new)
 {
 	struct images *imgs=&cnt->imgs;
 	int i;
@@ -217,13 +206,13 @@ void alg_noise_tune (struct context *cnt, unsigned char *new)
 	i=imgs->motionsize;
 	
 	for (; i>0; i--) {
-		diff=*ref-*new;
+		diff = *ref - *new;
 		
 		if (mask)
-			diff=((diff* *mask++)/255);
+			diff = ((diff * *mask++)/255);
 
 		if (*smartmask){
-			sum+=ABS(diff)+1;
+			sum += ABS(diff) + 1;
 			count++;
 		}
 
@@ -232,41 +221,41 @@ void alg_noise_tune (struct context *cnt, unsigned char *new)
 		smartmask++;
 	}
 	
-	if (count>4) /* avoid divide by zero */
-		sum/=count/4;
+	if (count > 4) /* avoid divide by zero */
+		sum /= count/4;
 		
-	cnt->noise=4+(cnt->noise+sum)/2;
+	cnt->noise = 4 + (cnt->noise+sum)/2;
 }
 
 void alg_threshold_tune(struct context *cnt, int diffs, int motion)
 {
 	int i;
-	int sum=0, top=diffs;
+	int sum = 0, top = diffs;
 
 	if (!diffs)
 		return;
 
 	if (motion)
-		diffs=cnt->threshold/4;
+		diffs = cnt->threshold / 4;
 
-	for (i=0; i<THRESHOLD_TUNE_LENGTH-1; i++)
+	for (i = 0; i < THRESHOLD_TUNE_LENGTH - 1; i++)
 	{
-		sum+=cnt->diffs_last[i];
+		sum += cnt->diffs_last[i];
 		if (cnt->diffs_last[i+1] && !motion)
-			cnt->diffs_last[i]=cnt->diffs_last[i+1];
+			cnt->diffs_last[i] = cnt->diffs_last[i+1];
 		else
-			cnt->diffs_last[i]=cnt->threshold/4;
-		if (cnt->diffs_last[i]>top)
-			top=cnt->diffs_last[i];
+			cnt->diffs_last[i] = cnt->threshold / 4;
+		if (cnt->diffs_last[i] > top)
+			top = cnt->diffs_last[i];
 	}
-	sum+=cnt->diffs_last[i];
-	cnt->diffs_last[i]=diffs;
+	sum += cnt->diffs_last[i];
+	cnt->diffs_last[i] = diffs;
 
-	sum/=THRESHOLD_TUNE_LENGTH/4;
-	if (sum<top*2)
-		sum=top*2;
+	sum /= THRESHOLD_TUNE_LENGTH / 4;
+	if (sum < top * 2)
+		sum = top * 2;
 	if (sum < cnt->conf.max_changes)
-			cnt->threshold=(cnt->threshold+sum)/2;
+		cnt->threshold = (cnt->threshold + sum) / 2;
 }
 
 /*
@@ -293,7 +282,7 @@ http://www.codeproject.com/gdi/QuickFill.asp
 typedef struct {short y, xl, xr, dy;} Segment;
 
 
-static int iflood(int x, int y,       /* seed point */
+static int iflood(int x, int y,       
                   int width, int height, unsigned char *out, int *labels, int newvalue, int oldvalue)
 {
 	int l, x1, x2, dy;
@@ -349,7 +338,7 @@ static int iflood(int x, int y,       /* seed point */
 	return count;
 }
 
-int alg_labeling (struct context *cnt)
+static int alg_labeling(struct context *cnt)
 {
 	struct images *imgs=&cnt->imgs;
 	unsigned char *out=imgs->out;
@@ -658,14 +647,20 @@ int alg_despeckle(struct context *cnt, int olddiffs)
 		case 'l':
 			diffs = alg_labeling(cnt);
 			i=len;
-			done=1;
+			done=2;
 			break;
 		}
 	}
-	if (done)
+
+	/* If conf.despeckle contains any valid action EeDdl */
+	if (done){
+		if (done != 2) cnt->imgs.labelsize_max = 0; // Disable Labeling
 		return diffs;
+	}	
 	else
-		return olddiffs;
+		cnt->imgs.labelsize_max = 0; // Disable Labeling
+	
+	return olddiffs;
 }
 
 /* Generate actual smartmask. Calculate sensitivity based on motion */
@@ -833,7 +828,7 @@ int alg_diff_standard (struct context *cnt, unsigned char *new)
 		}
 
 		/* Next, compare the multiplied absolute differences with the multiplied
-		 * noise level (repeted as 4 words in mm7), resulting in a "motion flag"
+		 * noise level (repeated as 4 words in mm7), resulting in a "motion flag"
 		 * for each pixel.
 		 *
 		 * Since pcmpgtw performs signed comparisons, we have to subtract noise,
@@ -939,18 +934,19 @@ int alg_diff_standard (struct context *cnt, unsigned char *new)
 	 */
 
 	for (; i>0; i--) {
-		register unsigned char curdiff=(int)(abs(*ref-*new)); /* using a temp variable is 12% faster */
+		register unsigned char curdiff=(int)(abs(*ref - *new)); /* using a temp variable is 12% faster */
 		/* apply fixed mask */
 		if (mask)
-			curdiff=((int)(curdiff**mask++)/255);
+			curdiff=((int)(curdiff * *mask++)/255);
+			
 		if (smartmask_speed) {
 			if (curdiff > noise) {
-				/* increase smart_mask sesitivity every frame when motion
+				/* increase smart_mask sensitivity every frame when motion
 				   is detected. (with speed=5, mask is increased by 1 every
 				   second. To be able to increase by 5 every second (with
 				   speed=10) we add 5 here. NOT related to the 5 at ratio-
 				   calculation. */
-				(*smartmask_buffer)+=SMARTMASK_SENSITIVITY_INCR;
+				(*smartmask_buffer) += SMARTMASK_SENSITIVITY_INCR;
 				/* apply smart_mask */
 				if (!*smartmask_final)
 					curdiff=0;
@@ -974,7 +970,7 @@ int alg_diff_standard (struct context *cnt, unsigned char *new)
 	Very fast diff function, does not do nightcompensation or mask
 	overlaying.
 */
-char alg_diff_fast (struct context *cnt, int max_n_changes, unsigned char *new)
+static char alg_diff_fast(struct context *cnt, int max_n_changes, unsigned char *new)
 {
 	struct images *imgs=&cnt->imgs;
 	int i, diffs=0, step=imgs->motionsize/10000;
@@ -1004,7 +1000,7 @@ char alg_diff_fast (struct context *cnt, int max_n_changes, unsigned char *new)
 /* alg_diff uses diff_fast to quickly decide if there is anything worth
  * sending to diff_standard.
 */
-int alg_diff (struct context *cnt, unsigned char *new)
+int alg_diff(struct context *cnt, unsigned char *new)
 {
 	int diffs=0;
 	
@@ -1018,7 +1014,7 @@ int alg_diff (struct context *cnt, unsigned char *new)
    It is assumed to be the light being switched on or a camera displacement.
    In any way the user doesn't think it is worth capturing.
  */
-int alg_lightswitch (struct context *cnt, int diffs)
+int alg_lightswitch(struct context *cnt, int diffs)
 {
 	struct images *imgs=&cnt->imgs;
 	
@@ -1034,34 +1030,34 @@ int alg_lightswitch (struct context *cnt, int diffs)
 	return 0;
 }
 
-int alg_switchfilter (struct context *cnt, int diffs, struct coord *cent, unsigned char *newimg)
+int alg_switchfilter(struct context *cnt, int diffs, unsigned char *newimg)
 {
 	int linediff = diffs / cnt->imgs.height;
 	unsigned char *out = cnt->imgs.out;
 	int y, x, line;
 	int lines=0, vertlines=0;
 
-	for (y=0; y<cnt->imgs.height; y++) {
+	for (y=0; y < cnt->imgs.height; y++) {
 		line=0;
-		for (x=0; x<cnt->imgs.width; x++) {
+		for (x=0; x < cnt->imgs.width; x++) {
 			if (*(out++)) {
 				line++;
 			}
 		}
-		if (line>cnt->imgs.width/18) {
+		if (line > cnt->imgs.width/18) {
 			vertlines++;
 		}
-		if (line>linediff*2) {
+		if (line > linediff*2) {
 			lines++;
 		}
 	}
-//	printf("%d %d %d %d  %d\n", lines, vertlines, linediff, lines-vertlines, diffs);
-	if (vertlines>cnt->imgs.height/10 && lines<vertlines/3 &&
-	    (vertlines>cnt->imgs.height/4 || lines-vertlines>lines/2)) {
+
+	if (vertlines > cnt->imgs.height/10 && lines < vertlines/3 &&
+	    (vertlines > cnt->imgs.height/4 || lines - vertlines > lines/2)) {
 		if (cnt->conf.text_changes) {
 			char tmp[80];
 			sprintf(tmp, "%d %d", lines, vertlines);
-			draw_text(newimg, cnt->imgs.width-10, 20, cnt->imgs.width, cnt->imgs.height, tmp, cnt->conf.text_double);
+			draw_text(newimg, cnt->imgs.width-10, 20, cnt->imgs.width, tmp, cnt->conf.text_double);
 		}
 		return diffs;
 	}
