@@ -31,12 +31,12 @@ struct segment {
 	int count;
 };
 
-struct coord alg_locate_center_size(struct images *, int width, int height);
-void alg_draw_location(struct coord *, struct images *, int width, int height, unsigned char *, int);
+void alg_locate_center_size(struct images *, int width, int height, struct coord *);
+void alg_draw_location(struct coord *, struct images *, int width, unsigned char *, int);
 int alg_diff(struct context *, unsigned char *);
 int alg_diff_standard(struct context *, unsigned char *);
 int alg_lightswitch(struct context *, int diffs);
-int alg_switchfilter(struct context *, int, struct coord *, unsigned char *);
+int alg_switchfilter(struct context *, int, unsigned char *);
 void alg_noise_tune(struct context *, unsigned char *);
 void alg_threshold_tune(struct context *, int, int);
 int alg_despeckle(struct context *, int);
