@@ -26,11 +26,11 @@
 #include "motion.h"
 
 
-#ifdef __freebsd__
+#if (defined(BSD))
 #include "video_freebsd.h"
 #else
 #include "video.h"
-#endif /* __freebsd__ */
+#endif /* BSD */
 
 
 #define stripnewline(x) {if ((x)[strlen(x)-1]=='\n') (x)[strlen(x) - 1] = 0; }
@@ -57,11 +57,10 @@ struct config conf_template = {
 	ppm:                   0,
 	noise:                 DEF_NOISELEVEL,
 	noise_tune:            1,
-	mingap:                0,
+	minimum_frame_time:    0,
 	lightswitch:           0,
 	nightcomp:             0,
 	low_cpu:               0,
-	nochild:               0,
 	autobright:            0,
 	brightness:            0,
 	contrast:              0,
@@ -91,7 +90,7 @@ struct config conf_template = {
 	tuner_number:          0,
 	timelapse:             0,
 	timelapse_mode:        DEF_TIMELAPSE_MODE,
-#ifdef  __freebsd__
+#if (defined(BSD))
 	tuner_device:          NULL,
 #endif
 	video_device:          VIDEO_DEVICE,
@@ -111,7 +110,7 @@ struct config conf_template = {
 	sql_log_timelapse:     0,
 	sql_query:             DEF_SQL_QUERY,
 	mysql_db:              NULL,
-	mysql_host:            NULL,
+	mysql_host:            "localhost",
 	mysql_user:            NULL,
 	mysql_password:        NULL,
 	on_picture_save:       NULL,
@@ -123,7 +122,7 @@ struct config conf_template = {
 	netcam_userpass:       NULL,
 	netcam_proxy:          NULL,
 	pgsql_db:              NULL,
-	pgsql_host:            NULL,
+	pgsql_host:            "localhost",
 	pgsql_user:            NULL,
 	pgsql_password:        NULL,
 	pgsql_port:            5432,
@@ -134,7 +133,7 @@ struct config conf_template = {
 	text_double:           0,
 	despeckle:             NULL,
 	minimum_motion_frames: 1,
-	// debug_parameter:       0
+	pid_file:              NULL,
 };
 
 
@@ -168,6 +167,13 @@ config_param config_params[] = {
 	print_bool
 	},
 	{
+	"process_id_file",
+	"#File to store the process ID, also called pid file. (default: not defined)",
+	CONF_OFFSET(pid_file),
+	copy_string,
+	print_string
+	},
+	{
 	"setup_mode",
 	"############################################################\n"
 	"# Basic Setup Mode\n"
@@ -188,7 +194,7 @@ config_param config_params[] = {
 	copy_string,
 	print_string
 	},
-#ifdef __freebsd__
+#if (defined(BSD))
 	{
 	"tunerdevice",
 	"# Tuner device to be used for capturing using tuner as source (default /dev/tuner0)\n"
@@ -252,8 +258,17 @@ config_param config_params[] = {
 	print_int
 	},
 	{
+	"minimum_frame_time",
+	"# Minimum time in seconds between capturing picture frames from the camera.\n"
+	"# Default: 0 = disabled - the capture rate is given by the camera framerate.\n"
+	"# This option is used when you want to capture images at a rate lower than 2 per second.",
+	CONF_OFFSET(minimum_frame_time),
+	copy_int,
+	print_int
+	},
+	{
 	"netcam_url",
-	"# URL to use if you are using a network camera, size will be autodetected (incl http://)\n"
+	"# URL to use if you are using a network camera, size will be autodetected (incl http:// ftp:// or file:///)\n"
 	"# Must be a URL that returns single jpeg pictures or a raw mjpeg stream. Default: Not defined",
 	CONF_OFFSET(netcam_url),
 	copy_string,
@@ -458,15 +473,6 @@ config_param config_params[] = {
 	print_int
 	},
 	{
-	"minimum_gap",
-	"# Minimum gap in seconds between the storing pictures while detecting motion.\n"
-	"# Default: 0 = as fast as possible (given by the camera framerate)\n"
-	"# Note: This option has nothing to do with the option 'gap'",
-	CONF_OFFSET(mingap),
-	copy_int,
-	print_int
-	},
-	{
 	"max_mpeg_time",
 	"# Maximum length in seconds of an mpeg movie\n"
 	"# When value is exceeded a new mpeg file is created. (Default: 0 = infinite)",
@@ -588,9 +594,12 @@ config_param config_params[] = {
 	"# Timelapse mpegs are always made in mpeg1 format independent from this option.\n"
 	"# Supported formats are: mpeg1 (ffmpeg-0.4.8 only), mpeg4 (default), and msmpeg4.\n"
 	"# mpeg1 - gives you files with extension .mpg\n"
-	"# mpeg4 or msmpeg4 - give you files with extension .avi\n"
+	"# mpeg4 or msmpeg4 - gives you files with extension .avi\n"
 	"# msmpeg4 is recommended for use with Windows Media Player because\n"
-	"# it requires no installation of codec on the Windows client.",
+	"# it requires no installation of codec on the Windows client.\n"
+	"# swf - gives you a flash film with extension .swf\n"
+	"# flv - gives you a flash video with extension .flv\n"
+	"# ffv1 - FF video codec 1 for Lossless Encoding ( experimental )",
 	CONF_OFFSET(ffmpeg_video_codec),
 	copy_string,
 	print_string
@@ -843,7 +852,7 @@ config_param config_params[] = {
 	"\n############################################################\n"
 	"# Tracking (Pan/Tilt)\n"
 	"############################################################\n\n"
-	"# Type of tracker (0=none (default), 1=stepper, 2=iomojo, 3=pwc, 4=generic)\n"
+	"# Type of tracker (0=none (default), 1=stepper, 2=iomojo, 3=pwc, 4=generic, 5=uvcvideo)\n"
 	"# The generic type enables the definition of motion center and motion size to\n"
 	"# be used with the conversion specifiers for options like on_motion_detected",
 	TRACK_OFFSET(type),
@@ -1157,19 +1166,6 @@ config_param config_params[] = {
 	copy_string,
 	print_string
 	},
-/*
-	{
-	"debug_parameter",
-	"\n############################################################\n"
-	"# Thread Config Files - One for each camera\n"
-	"# If Only One Camera - Use default values in this file\n"
-	"############################################################\n\n",
-	//"# Debug option for programmers - not normally used",
-	CONF_OFFSET(debug_parameter),
-	copy_int,
-	print_int
-	},
-*/
 	{
 	"thread",
 	"\n##############################################################\n"
@@ -1197,7 +1193,7 @@ static void conf_cmdline (struct context *cnt, int thread)
 	 * if necessary. This is accomplished by calling mystrcpy();
 	 * see this function for more information.
 	 */
-	while ((c=getopt(conf->argc, conf->argv, "c:d:hns?"))!=EOF)
+	while ((c=getopt(conf->argc, conf->argv, "c:d:hns?p:"))!=EOF)
 		switch (c) {
 			case 'c':
 				if (thread==-1) strcpy(cnt->conf_filename, optarg);
@@ -1212,6 +1208,9 @@ static void conf_cmdline (struct context *cnt, int thread)
 				/* no validation - just take what user gives */
 				debug_level = atoi(optarg);
 				break;
+			case 'p':
+				if (thread==-1) strcpy(cnt->pid_file, optarg);
+				break;	
 			case 'h':
 			case '?':
 			default:
@@ -1319,8 +1318,8 @@ static struct context **conf_process(struct context **cnt, FILE *fp)
 			/* trim space between command and argument */
 			beg++;
 
-			if (strlen(beg) > 0) {
-				while (*beg == ' ' || *beg == '\t' || *beg == '=') {
+			if (strlen(beg) > 0){
+				while (*beg == ' ' || *beg == '\t' || *beg == '=' || *beg == '\n' || *beg == '\r') {
 					beg++;
 				}
 
@@ -1452,8 +1451,10 @@ struct context ** conf_load (struct context **cnt)
 	 * 3. $HOME/.motion/motion.conf
 	 * 4. sysconfig/motion.conf
 	 */
-	/* Get filename from commandline */
+	/* Get filename & pid file from commandline */
 	cnt[0]->conf_filename[0] = 0;
+	cnt[0]->pid_file[0] = 0;
+
 	conf_cmdline(cnt[0], -1);
 	if (cnt[0]->conf_filename[0]){ /* User has supplied filename on commandline*/
 		strcpy(filename, cnt[0]->conf_filename);
@@ -1466,11 +1467,11 @@ struct context ** conf_load (struct context **cnt)
 		fp = fopen (filename, "r");
 	}
 	if (!fp) {     /* specified file does not exist... try default file */
-		sprintf(filename, "%s/.motion/motion.conf", getenv("HOME"));
-		fp = fopen (filename, "r");
+		snprintf(filename, PATH_MAX, "%s/.motion/motion.conf", getenv("HOME"));
+		fp = fopen(filename, "r");
 		if (!fp) {
-			sprintf(filename, "%s/motion.conf", sysconfdir);
-			fp = fopen (filename, "r");
+			snprintf(filename, PATH_MAX, "%s/motion.conf", sysconfdir);
+			fp = fopen(filename, "r");
 			if (!fp)        /* there is no config file.... use defaults */
 				motion_log(-1, 1, "could not open configfile %s",filename);
 		}
@@ -1482,6 +1483,8 @@ struct context ** conf_load (struct context **cnt)
 		motion_log(LOG_INFO, 0, "Processing thread 0 - config file %s",filename);
 		cnt=conf_process(cnt, fp);
 		fclose(fp);
+	}else{
+		motion_log(LOG_INFO, 0, "Not config file to process using default values");
 	}
 	
 
@@ -1497,6 +1500,10 @@ struct context ** conf_load (struct context **cnt)
 	while(cnt[++i])
 		conf_cmdline(cnt[i], i);
 
+	/* if pid file was passed from command line copy to main thread conf struct */
+	if (cnt[0]->pid_file[0])
+		cnt[0]->conf.pid_file = mystrcpy(cnt[0]->conf.pid_file, cnt[0]->pid_file);
+
 	return cnt;
 }
 
@@ -1837,11 +1844,12 @@ static void usage ()
 	printf("\nusage:\tmotion [options]\n");
 	printf("\n\n");
 	printf("Possible options:\n\n");
-	printf("-n\t\tRun in non-daemon mode.\n");
-	printf("-s\t\tRun in setup mode.\n");
-	printf("-c config\tFull path and filename of config file.\n");
+	printf("-n\t\t\tRun in non-daemon mode.\n");
+	printf("-s\t\t\tRun in setup mode.\n");
+	printf("-c config\t\tFull path and filename of config file.\n");
 	printf("-d level\t\tDebug mode.\n");
-	printf("-h\t\tShow this screen.\n");
+	printf("-p process_id_file\tFull path and filename of process id file (pid file).\n");
+	printf("-h\t\t\tShow this screen.\n");
 	printf("\n");
 	printf("Motion is configured using a config file only. If none is supplied,\n");
 	printf("it will read motion.conf from current directory, ~/.motion or %s.\n", sysconfdir);
