@@ -12,22 +12,37 @@
 #include <ctype.h>
 #include "motion.h"
 
-/* The number of characters in draw_table. */
-#define NUM_CHARS 81
+/* highest ascii value is 126 (~) */
+#define ASCII_MAX 127
+
+unsigned char *small_char_arr_ptr[ASCII_MAX];
+
+unsigned char *big_char_arr_ptr[ASCII_MAX];
 
 struct draw_char {
-	char ascii;
-	char pix[8][7];
+	unsigned char ascii;
+	unsigned char pix[8][7];
 };
 
 struct big_char {
-	char ascii;
-	char pix[16][14];
+	unsigned char ascii;
+	unsigned char pix[16][14];
 };
 
-struct big_char big_table[NUM_CHARS];
-
-struct draw_char draw_table[NUM_CHARS]= {
+struct draw_char draw_table[]= {
+	{
+		' ',
+		{
+			{0,0,0,0,0,0,0},
+			{0,0,0,0,0,0,0},
+			{0,0,0,0,0,0,0},
+			{0,0,0,0,0,0,0},
+			{0,0,0,0,0,0,0},
+			{0,0,0,0,0,0,0},
+			{0,0,0,0,0,0,0},
+			{0,0,0,0,0,0,0}
+		}
+	},
 	{
 		'0',
 		{
@@ -1054,104 +1069,59 @@ struct draw_char draw_table[NUM_CHARS]= {
 			{1,2,2,2,2,2,1},
 			{0,1,1,1,1,1,0}
 		}
-	},
-	{
-		' ',
-		{
-			{0,0,0,0,0,0,0},
-			{0,0,0,0,0,0,0},
-			{0,0,0,0,0,0,0},
-			{0,0,0,0,0,0,0},
-			{0,0,0,0,0,0,0},
-			{0,0,0,0,0,0,0},
-			{0,0,0,0,0,0,0},
-			{0,0,0,0,0,0,0}
-		}
-	},
-	{
-		0,
-		{
-			{0,0,0,0,0,0,0},
-			{0,0,0,0,0,0,0},
-			{0,0,0,0,0,0,0},
-			{0,0,0,0,0,0,0},
-			{0,0,0,0,0,0,0},
-			{0,0,0,0,0,0,0},
-			{0,0,0,0,0,0,0},
-			{0,0,0,0,0,0,0}
-		}
 	}
 };
 
+struct big_char big_table[sizeof(draw_table) / sizeof(struct draw_char)];
+
 #define NEWLINE "\\n"
 
-static int draw_textn (unsigned char *image, int startx, int starty, int width, int height, char *text, int len, int factor)
+static int draw_textn (unsigned char *image, int startx, int starty, int width, char *text, int len, int factor)
 {
-	int i, pos, x, y , tempvar;
+	int pos, x, y, line_offset, next_char_offs;
+	unsigned char *image_ptr, *char_ptr, **char_arr_ptr;
+
+	if (startx>width/2)
+		startx -= len*(6*(factor+1));
 
 	if (startx < 0)
 		startx = 0;
-	if (startx>width/2) {
-		startx -= len*(6*(factor+1));
-	}
-	/* Draw text at normal (small) size. */
-	if(!factor) {
-		for (pos = 0; pos < len; pos++) {
-			if (startx+(pos+1)*6>=width)
-				break;
-			i=0;
-			tempvar=startx+pos*6;
-			
-			while (draw_table[i].ascii && draw_table[i].ascii!=text[pos]) i++;
-			
-			for (x=0; x<7; x++) {
-				int tempvar_x = tempvar + x;
-				for (y=0; y<8; y++) {
-					switch(draw_table[i].pix[y][x]) {
-						case 1:
-							image[tempvar_x+(y+starty)*width]=0;
-							break;
-						case 2:
-							image[tempvar_x+(y+starty)*width]=255;
-							break;
-						default:
-							break;
-					}
-				}
-			}
-		}
-	}
-	/* Draw text at double normal size. */
-	else {
-		for (pos = 0; pos < len; pos++) {
-			if (startx+(pos+1)*12>=width)
-				break;
-			i=0;
-			tempvar=startx+pos*12;
-			
-			while (big_table[i].ascii && big_table[i].ascii!=text[pos]) i++;
-			
-			for (x=0; x<14; x++) {
-				int tempvar_x = tempvar + x;
-				for (y=0; y<16; y++) {
-					switch(big_table[i].pix[y][x]) {
-						case 1:
-							image[tempvar_x+(y+starty)*width]=0;
-							break;
-						case 2:
-							image[tempvar_x+(y+starty)*width]=255;
-							break;
-						default:
-							break;
-					}
+	
+	if (startx+len*6*(factor+1) >= width)
+		len = (width-startx-1)/(6*(factor+1));
+	
+	line_offset = width - 7*(factor+1);
+	next_char_offs = width*8*(factor+1) - 6*(factor+1);
+	
+	image_ptr = image + startx + starty*width;
+
+	char_arr_ptr = factor ? big_char_arr_ptr : small_char_arr_ptr;
+	
+	for (pos = 0; pos < len; pos++) {
+		char_ptr = char_arr_ptr[(int)text[pos]];
+		for (y=8*(factor+1); y--;) {
+			for (x=7*(factor+1); x--;) {
+				switch(*char_ptr) {
+					case 1:
+						*image_ptr = 0;
+						break;
+					case 2:
+						*image_ptr = 255;
+						break;
+					default:
+						break;
 				}
+				image_ptr++;
+				char_ptr++;
 			}
+			image_ptr += line_offset;
 		}
+		image_ptr -= next_char_offs;
 	}
 	return 0;
 }
 
-int draw_text (unsigned char *image, int startx, int starty, int width, int height, char *text, int factor)
+int draw_text (unsigned char *image, int startx, int starty, int width, char *text, int factor)
 {
 	int num_nl = 0;
 	char *end, *begin;
@@ -1168,18 +1138,18 @@ int draw_text (unsigned char *image, int startx, int starty, int width, int heig
 	begin = end = text;
 	while ((end = strstr(end, NEWLINE))) {
 		int len = end-begin;
-		draw_textn(image, startx, starty, width, height, begin, len, factor);
+		draw_textn(image, startx, starty, width, begin, len, factor);
 		end += sizeof(NEWLINE)-1;
 		begin = end;
 		starty += line_space;
 	}
-	draw_textn(image, startx, starty, width, height, begin, strlen(begin), factor);
+	draw_textn(image, startx, starty, width, begin, strlen(begin), factor);
 	return 0;
 }
 
 int initialize_chars(void)
 {
-	int i=0, x, y;
+	unsigned int i=0, x, y;
 	
 	/* Fill the structure 'big_table' with double sized characters. */
 	while(draw_table[i].ascii) {
@@ -1191,7 +1161,18 @@ int initialize_chars(void)
 		}
 		i++;
 	}
-	
+
+	/* first init all char ptr's to a space character */
+	for (i=0; i < ASCII_MAX; i++) {
+		small_char_arr_ptr[i]=&draw_table[0].pix[0][0];
+		big_char_arr_ptr[i]=&big_table[0].pix[0][0];
+	}
+			
+	/* build [big_]char_arr_ptr table to point to each available ascii */
+	for (i=0; i < sizeof(draw_table) / sizeof(struct draw_char); i++) {
+		small_char_arr_ptr[(int)draw_table[i].ascii]=&draw_table[i].pix[0][0];
+		big_char_arr_ptr[(int)draw_table[i].ascii]=&big_table[i].pix[0][0];
+	}
 	return 0;
 }
 
