@@ -12,7 +12,7 @@
 #include "ffmpeg.h"           /* must be first to avoid 'shadow' warning */
 #include "motion.h"
 #include "event.h"
-#ifndef __freebsd__
+#if (!defined(BSD)) 
 #include "video.h"
 #endif
 #include "picture.h"
@@ -125,6 +125,7 @@ static void event_sqlnewfile(struct context *cnt, int type  ATTRIBUTE_UNUSED,
 		char sqlquery[PATH_MAX];
 		char timestr[20];
 		char eventtimestr[20];
+		int ret;
 	
 		strftime(timestr, sizeof(timestr), "%Y-%m-%d %T", cnt->currenttime_tm);
 		strftime(eventtimestr, sizeof(eventtimestr), "%Y-%m-%d %T", cnt->eventtime_tm);
@@ -133,8 +134,25 @@ static void event_sqlnewfile(struct context *cnt, int type  ATTRIBUTE_UNUSED,
 
 #ifdef HAVE_MYSQL
 		if (cnt->conf.mysql_db) {
-			if (mysql_query(cnt->database, sqlquery) != 0)
-				motion_log(LOG_ERR, 1, "Mysql query failed");
+			ret = mysql_query(cnt->database, sqlquery);
+			if (ret != 0){
+				int error_code = mysql_errno(cnt->database);
+				
+				motion_log(LOG_ERR, 1, "Mysql query failed %s error code %d",
+						mysql_error(cnt->database), error_code);
+				/* Try to reconnect ONCE if fails continue and discard this sql query */
+				if (error_code >= 2000){
+					cnt->database = (MYSQL *) mymalloc(sizeof(MYSQL));
+					mysql_init(cnt->database);
+					if (!mysql_real_connect(cnt->database, cnt->conf.mysql_host, 
+								cnt->conf.mysql_user, cnt->conf.mysql_password, 
+								cnt->conf.mysql_db, 0, NULL, 0)) {
+						motion_log(LOG_ERR, 0, "Cannot reconnect to MySQL database %s on host %s with user %s", cnt->conf.mysql_db, cnt->conf.mysql_host, cnt->conf.mysql_user);
+						motion_log(LOG_ERR, 0, "MySQL error was %s", mysql_error(cnt->database));
+					}else mysql_query(cnt->database, sqlquery);
+				}	
+				
+			}	
 		}
 #endif /* HAVE_MYSQL */
 
@@ -194,7 +212,7 @@ static void event_webcam_put(struct context *cnt, int type ATTRIBUTE_UNUSED,
 }
 
 #ifndef WITHOUT_V4L
-#ifndef __freebsd__
+#if (!defined(BSD))
 static void event_vid_putpipe(struct context *cnt, int type ATTRIBUTE_UNUSED,
             unsigned char *img, char *dummy ATTRIBUTE_UNUSED, void *devpipe,
             struct tm *tm ATTRIBUTE_UNUSED)
@@ -204,7 +222,7 @@ static void event_vid_putpipe(struct context *cnt, int type ATTRIBUTE_UNUSED,
 			motion_log(LOG_ERR, 1, "Failed to put image into video pipe");
 	}
 }
-#endif /* __freebsd__ */
+#endif /* BSD */
 #endif /* WITHOUT_V4L */
 
 
@@ -237,9 +255,9 @@ static void event_image_detect(struct context *cnt, int type ATTRIBUTE_UNUSED,
 			
 		mystrftime(cnt, filename, sizeof(filename), jpegpath, currenttime_tm, NULL, 0);
 		/* motion images gets same name as normal images plus an appended 'm' */
-		sprintf(filenamem, "%sm", filename);
-		sprintf(fullfilename, "%s/%s.%s", cnt->conf.filepath, filename, imageext(cnt));
-		sprintf(fullfilenamem, "%s/%s.%s", cnt->conf.filepath, filenamem, imageext(cnt));
+		snprintf(filenamem, PATH_MAX, "%sm", filename);
+		snprintf(fullfilename, PATH_MAX, "%s/%s.%s", cnt->conf.filepath, filename, imageext(cnt));
+		snprintf(fullfilenamem, PATH_MAX, "%s/%s.%s", cnt->conf.filepath, filenamem, imageext(cnt));
 	}
 	if (conf->motion_img) {
 		put_picture(cnt, fullfilenamem, cnt->imgs.out, FTYPE_IMAGE_MOTION);
@@ -268,20 +286,20 @@ static void event_image_snapshot(struct context *cnt, int type ATTRIBUTE_UNUSED,
 			snappath = DEF_SNAPPATH;
 			
 		mystrftime(cnt, filepath, sizeof(filepath), snappath, currenttime_tm, NULL, 0);
-		sprintf(filename, "%s.%s", filepath, imageext(cnt));
-		sprintf(fullfilename, "%s/%s", cnt->conf.filepath, filename);
+		snprintf(filename, PATH_MAX, "%s.%s", filepath, imageext(cnt));
+		snprintf(fullfilename, PATH_MAX, "%s/%s", cnt->conf.filepath, filename);
 		put_picture(cnt, fullfilename, img, FTYPE_IMAGE_SNAPSHOT);
 
 		/* Update symbolic link *after* image has been written so that
 		   the link always points to a valid file. */
-		sprintf(linkpath, "%s/lastsnap.%s", cnt->conf.filepath, imageext(cnt));
+		snprintf(linkpath, PATH_MAX, "%s/lastsnap.%s", cnt->conf.filepath, imageext(cnt));
 		remove(linkpath);
 		if (symlink(filename, linkpath)) {
 			motion_log(LOG_ERR, 1, "Could not create symbolic link [%s]", filename);
 			return;
 		}
 	} else {
-		sprintf(fullfilename, "%s/lastsnap.%s", cnt->conf.filepath, imageext(cnt));
+		snprintf(fullfilename, PATH_MAX, "%s/lastsnap.%s", cnt->conf.filepath, imageext(cnt));
 		remove(fullfilename);
 		put_picture(cnt, fullfilename, img, FTYPE_IMAGE_SNAPSHOT);
 	}
@@ -330,8 +348,9 @@ static void event_ffmpeg_newfile(struct context *cnt, int type ATTRIBUTE_UNUSED,
 	mystrftime(cnt, stamp, sizeof(stamp), mpegpath, currenttime_tm, NULL, 0);
 
 	/* motion mpegs get the same name as normal mpegs plus an appended 'm' */
-	sprintf(cnt->motionfilename, "%s/%sm", cnt->conf.filepath, stamp);
-	sprintf(cnt->newfilename, "%s/%s", cnt->conf.filepath, stamp);
+	/* PATH_MAX - 4 to allow for .mpg to be appended without overflow */
+	snprintf(cnt->motionfilename, PATH_MAX - 4, "%s/%sm", cnt->conf.filepath, stamp);
+	snprintf(cnt->newfilename, PATH_MAX - 4, "%s/%s", cnt->conf.filepath, stamp);
 
 	if (cnt->conf.ffmpeg_cap_new) {
 		if (cnt->imgs.type==VIDEO_PALETTE_GREY) {
@@ -358,7 +377,7 @@ static void event_ffmpeg_newfile(struct context *cnt, int type ATTRIBUTE_UNUSED,
 		      ffmpeg_open((char *)cnt->conf.ffmpeg_video_codec, cnt->newfilename, y, u, v,
 		                  cnt->imgs.width, cnt->imgs.height, fps, cnt->conf.ffmpeg_bps,
 		                  cnt->conf.ffmpeg_vbr)) == NULL) {
-			motion_log(LOG_ERR, 1, "ffopen_open error creating file [%s]",cnt->newfilename);
+			motion_log(LOG_ERR, 1, "ffopen_open error creating (new) file [%s]",cnt->newfilename);
 			cnt->finish=1;
 			return;
 		}
@@ -390,7 +409,7 @@ static void event_ffmpeg_newfile(struct context *cnt, int type ATTRIBUTE_UNUSED,
 		      ffmpeg_open((char *)cnt->conf.ffmpeg_video_codec, cnt->motionfilename, y, u, v,
 		                  cnt->imgs.width, cnt->imgs.height, fps, cnt->conf.ffmpeg_bps,
 		                  cnt->conf.ffmpeg_vbr)) == NULL){
-			motion_log(LOG_ERR, 1, "ffopen_open error creating file [%s]", cnt->motionfilename);
+			motion_log(LOG_ERR, 1, "ffopen_open error creating (motion) file [%s]", cnt->motionfilename);
 			cnt->finish=1;
 			return;
 		}
@@ -420,7 +439,9 @@ static void event_ffmpeg_timelapse(struct context *cnt,
 			timepath = DEF_TIMEPATH;
 		
 		mystrftime(cnt, tmp, sizeof(tmp), timepath, currenttime_tm, NULL, 0);
-		sprintf(cnt->timelapsefilename, "%s/%s", cnt->conf.filepath, tmp);
+		
+		/* PATH_MAX - 4 to allow for .mpg to be appended without overflow */
+		snprintf(cnt->timelapsefilename, PATH_MAX - 4, "%s/%s", cnt->conf.filepath, tmp);
 		
 		if (cnt->imgs.type == VIDEO_PALETTE_GREY) {
 			convbuf = mymalloc((width*height)/2);
@@ -439,7 +460,7 @@ static void event_ffmpeg_timelapse(struct context *cnt,
 		      ffmpeg_open((char *)TIMELAPSE_CODEC, cnt->timelapsefilename, y, u, v,
 		                  cnt->imgs.width, cnt->imgs.height, 24, cnt->conf.ffmpeg_bps,
 		                  cnt->conf.ffmpeg_vbr)) == NULL) {
-			motion_log(LOG_ERR, 1, "ffopen_open error creating file [%s]", cnt->timelapsefilename);
+			motion_log(LOG_ERR, 1, "ffopen_open error creating (timelapse) file [%s]", cnt->timelapsefilename);
 			cnt->finish=1;
 			return;
 		}
@@ -577,12 +598,12 @@ struct event_handlers event_handlers[] = {
 	event_image_snapshot
 	},
 #ifndef WITHOUT_V4L
-#ifndef __freebsd__
+#if (!defined(BSD))
 	{
 	EVENT_IMAGE | EVENT_IMAGEM,
 	event_vid_putpipe
 	},
-#endif /* __freebsd__ */
+#endif /* BSD */
 #endif /* WITHOUT_V4L */
 	{
 	EVENT_WEBCAM,
