@@ -32,6 +32,6 @@
 typedef void(* event_handler)(struct context *, int, unsigned char *, char *, void *, struct tm *);
 
 void event(struct context *, int, unsigned char *, char *, void *, struct tm *);
-char * imageext(struct context *);
+const char * imageext(struct context *);
 
 #endif /* _INCLUDE_EVENT_H_ */
