@@ -15,8 +15,6 @@
 #include "ffmpeg.h"
 #include "motion.h"
 
-
-
 #if LIBAVCODEC_BUILD > 4680
 /* FFmpeg after build 4680 doesn't have support for mpeg1 videos with 
  * non-standard framerates. Previous builds contained a broken hack 
@@ -38,13 +36,13 @@
 #	endif /* __GNUC__ */
 #endif /* LIBAVCODEC_BUILD > 4680 */
 
-#if FFMPEG_VERSION_INT >= 0x000409
+#if LIBAVFORMAT_BUILD >= 4616
 /* The API for av_write_frame changed with FFmpeg version 0.4.9pre1.
  * It now uses an AVPacket struct instead of direct parameters to the
- * function.
+ * function. The
  */
 #	define FFMPEG_AVWRITEFRAME_NEWAPI
-#endif /* FFMPEG_VERSION_INT >= 0x000409 */
+#endif /* LIBAVFORMAT_BUILD >= 4616 */
 
 #if LIBAVFORMAT_BUILD >= 4629
 /* In this build/header version, the codec member of struct AVStream
@@ -115,6 +113,76 @@ URLProtocol mpeg1_file_protocol = {
 	.url_open = file_open_append
 };
 
+
+#ifdef HAVE_FFMPEG_NEW
+
+/*
+ * file_procotol has been removed from avio.h
+ * 
+ */
+
+#include "avstring.h"
+
+static int file_open(URLContext *h, const char *filename, int flags)
+{
+    int access;
+    int fd;
+
+    av_strstart(filename, "file:", &filename);
+
+    if (flags & URL_RDWR) {
+        access = O_CREAT | O_TRUNC | O_RDWR;
+    } else if (flags & URL_WRONLY) {
+        access = O_CREAT | O_TRUNC | O_WRONLY;
+    } else {
+        access = O_RDONLY;
+    }
+#ifdef O_BINARY
+    access |= O_BINARY;
+#endif
+    fd = open(filename, access, 0666);
+    if (fd < 0)
+        return AVERROR(ENOENT);
+    h->priv_data = (void *)(size_t)fd;
+    return 0;
+}
+
+static int file_read(URLContext *h, unsigned char *buf, int size)
+{
+    int fd = (size_t)h->priv_data;
+    return read(fd, buf, size);
+}
+
+static int file_write(URLContext *h, unsigned char *buf, int size)
+{
+    int fd = (size_t)h->priv_data;
+    return write(fd, buf, size);
+}
+
+static offset_t file_seek(URLContext *h, offset_t pos, int whence)
+{
+    int fd = (size_t)h->priv_data;
+    return lseek(fd, pos, whence);
+}
+
+static int file_close(URLContext *h)
+{
+    int fd = (size_t)h->priv_data;
+    return close(fd);
+}
+
+URLProtocol file_protocol = {
+    "file",
+    file_open,
+    file_read,
+    file_write,
+    file_seek,
+    file_close,
+};
+
+#endif
+
+
 /* We set AVOutputFormat->write_trailer to this function for mpeg1. That way,
  * the mpeg1 video gets a proper trailer when it is closed.
  */
@@ -128,8 +196,11 @@ static int mpeg1_write_trailer(AVFormatContext *s)
 /* ffmpeg_init initializes for libavformat. */
 void ffmpeg_init()
 {
+	motion_log(LOG_INFO, 0, "ffmpeg LIBAVCODEC_BUILD %d LIBAVFORMAT_BUILD %d", LIBAVCODEC_BUILD, LIBAVFORMAT_BUILD);
 	av_register_all();
-
+#if LIBAVCODEC_BUILD > 4680	
+	av_log_set_callback( (void *)ffmpeg_avcodec_log );
+#endif
 	/* Copy the functions to use for the append file protocol from the standard
 	 * file protocol.
 	 */
@@ -161,7 +232,7 @@ static AVOutputFormat *get_oformat(const char *codec, char *filename)
 		|| (strcmp(codec, "mpeg1") == 0)
 #endif 
 	) {
-		ext = "mpg";
+		ext = ".mpg";
 		/* We use "mpeg1video" for raw mpeg1 format. Using "mpeg" would
 		 * result in a muxed output file, which isn't appropriate here.
 		 */
@@ -176,15 +247,29 @@ static AVOutputFormat *get_oformat(const char *codec, char *filename)
 		return NULL;
 #endif
 	} else if (strcmp(codec, "mpeg4") == 0) {
-		ext = "avi";
+		ext = ".avi";
 		of = guess_format("avi", NULL, NULL);
 	} else if (strcmp(codec, "msmpeg4") == 0) {
-		ext = "avi";
+		ext = ".avi";
 		of = guess_format("avi", NULL, NULL);
 		if (of) {
 			/* Manually override the codec id. */
 			of->video_codec = CODEC_ID_MSMPEG4V2;
 		}
+	} else if (strcmp(codec, "swf") == 0) {
+		ext = ".swf";
+		of = guess_format("swf", NULL, NULL);
+	} else if (strcmp(codec, "flv") == 0) {
+		ext = ".flv";
+		of = guess_format("flv", NULL, NULL);
+	} else if (strcmp(codec, "ffv1") == 0) {
+		ext = ".avi";
+		of = guess_format("avi", NULL, NULL);
+		if (of) {
+			/* Use the FFMPEG Lossless Video codec (experimental!).
+			Requires strict_std_compliance to be <= -2 */
+			of->video_codec = CODEC_ID_FLV1;
+		}
 	} else {
 		motion_log(LOG_ERR, 0, "ffmpeg_video_codec option value %s is not supported", codec);
 		return NULL;
@@ -195,8 +280,9 @@ static AVOutputFormat *get_oformat(const char *codec, char *filename)
 		return NULL;
 	}
 
-	/* WARNING: potential buffer overflow */
-	sprintf(filename, "%s.%s", filename, ext);
+	/* The 4 allows for ".avi" or ".mpg" to be appended */
+	strncat(filename, ext, 4);
+
 	return of;
 }
 
@@ -268,6 +354,9 @@ struct ffmpeg *ffmpeg_open(char *ffmpeg_video_codec, char *filename,
 	/* Uncomment to allow non-standard framerates. */
 	//c->strict_std_compliance = -1;
 
+	if (strcmp(ffmpeg_video_codec, "ffv1") == 0)
+		c->strict_std_compliance = -2;
+
 	/* Set default parameters */
 	c->bit_rate = bps;
 	c->width    = width;
@@ -317,13 +406,22 @@ struct ffmpeg *ffmpeg_open(char *ffmpeg_video_codec, char *filename,
 	/* Set the picture format - need in ffmpeg starting round April-May 2005 */
 	c->pix_fmt = PIX_FMT_YUV420P;
 
+	/* Get a mutex lock. */
+	pthread_mutex_lock(&global_lock);
+
 	/* open the codec */
 	if (avcodec_open(c, codec) < 0) {
+		/* Release the lock. */
+		pthread_mutex_unlock(&global_lock);
 		motion_log(LOG_ERR, 1, "avcodec_open - could not open codec");
 		ffmpeg_cleanups(ffmpeg);
 		return (NULL);
 	}
 
+	/* Release the lock. */
+	pthread_mutex_unlock(&global_lock);
+
+
 	ffmpeg->video_outbuf = NULL;
 	if (!(ffmpeg->oc->oformat->flags & AVFMT_RAWPICTURE)) {
 		/* allocate output buffer */
@@ -412,9 +510,14 @@ void ffmpeg_cleanups(struct ffmpeg *ffmpeg)
 
 	/* close each codec */
 	if (ffmpeg->video_st) {
-		avcodec_close(AVSTREAM_CODEC_PTR(ffmpeg->video_st));
+		pthread_mutex_lock(&global_lock);
+#if LIBAVCODEC_BUILD > 4680
+		if (ffmpeg->video_st->codec->priv_data != NULL)
+#endif
+			avcodec_close(AVSTREAM_CODEC_PTR(ffmpeg->video_st));
+		pthread_mutex_unlock(&global_lock);	
 		av_freep(&ffmpeg->picture);
-		av_freep(&ffmpeg->video_outbuf);
+		free(ffmpeg->video_outbuf);
 	}
 
 	/* free the streams */
@@ -430,7 +533,7 @@ void ffmpeg_cleanups(struct ffmpeg *ffmpeg)
 	/* free the stream */
 	av_free(ffmpeg->oc);
 #if LIBAVFORMAT_BUILD >= 4629
-	free(ffmpeg->c);
+	av_free(ffmpeg->c);
 #endif
 	free(ffmpeg);
 }
@@ -442,9 +545,11 @@ void ffmpeg_close(struct ffmpeg *ffmpeg)
 
 	/* close each codec */
 	if (ffmpeg->video_st) {
+		pthread_mutex_lock(&global_lock);
 		avcodec_close(AVSTREAM_CODEC_PTR(ffmpeg->video_st));
+		pthread_mutex_unlock(&global_lock);
 		av_freep(&ffmpeg->picture);
-		av_freep(&ffmpeg->video_outbuf);
+		free(ffmpeg->video_outbuf);
 	}
 
 	/* write the trailer, if any */
@@ -463,7 +568,7 @@ void ffmpeg_close(struct ffmpeg *ffmpeg)
 	/* free the stream */
 	av_free(ffmpeg->oc);
 #if LIBAVFORMAT_BUILD >= 4629
-	free(ffmpeg->c);
+	av_free(ffmpeg->c);
 #endif
 	free(ffmpeg);
 }
@@ -484,7 +589,7 @@ void ffmpeg_put_other_image(struct ffmpeg *ffmpeg, unsigned char *y,
 
 	if (picture) {
 		ffmpeg_put_frame(ffmpeg, picture);
-		free(picture);
+		av_free(picture);
 	}
 }
 
@@ -618,4 +723,31 @@ void ffmpeg_deinterlace(unsigned char *img, int width, int height)
 	return;
 }
 
+/** ffmpeg_avcodec_log
+ *      Handle any logging output from the ffmpeg library avcodec.
+ * 
+ * Parameters
+ *      *ignoreme  A pointer we will ignore
+ *      errno_flag The error number value
+ *      fmt        Text message to be used for log entry in printf() format.
+ *      ap         List of variables to be used in formatted message text.
+ *
+ * Returns
+ *      Function returns nothing.
+ */
+void ffmpeg_avcodec_log(void *ignoreme ATTRIBUTE_UNUSED, int errno_flag, const char *fmt, va_list vl)
+{
+	char buf[1024];
+
+	/* Do not log the message coming from avcodec if the debug_level is not set. */
+	if (debug_level) {
+
+		/* Flatten the message coming in from avcodec */
+		vsnprintf(buf, sizeof(buf), fmt, vl);
+
+		/* If the debug_level is correct then send the message to the motion logging routine. */
+		motion_log(LOG_ERR, 0, "ffmpeg_avcodec_log: %s - flag %d", buf, errno_flag);
+	}
+}
+
 #endif /* HAVE_FFMPEG */
