@@ -5,6 +5,9 @@
 #include <avformat.h>
 #endif
 
+#include <stdio.h>
+#include <stdarg.h>
+
 /* Define a codec name/identifier for timelapse videos, so that we can
  * differentiate between normal mpeg1 videos and timelapse videos.
  */
@@ -66,4 +69,7 @@ void ffmpeg_close(struct ffmpeg *);
 /*Deinterlace the image. */
 void ffmpeg_deinterlace(unsigned char *, int, int);
 
+/*Setup an avcodec log handler. */
+void ffmpeg_avcodec_log(void *, int, const char *, va_list);
+
 #endif /* _INCLUDE_FFMPEG_H_ */
