@@ -11,7 +11,6 @@
 #define _INCLUDE_MOTION_H
 
 /* Includes */
-
 #ifdef HAVE_MYSQL
 #include <mysql.h>
 #endif
@@ -33,9 +32,10 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <sys/ioctl.h>
+#include <sys/param.h>
 
 #define _LINUX_TIME_H 1
-#if (!defined(WITHOUT_V4L)) && (!defined (__freebsd__))
+#if (!defined(WITHOUT_V4L)) && (!defined(BSD))
 #include <linux/videodev.h>
 #endif
 
@@ -76,7 +76,7 @@
         } 
 
 
-#if defined(WITHOUT_V4L) || defined(__freebsd__)
+#if defined(WITHOUT_V4L) || defined(BSD)
  
 #define VIDEO_PALETTE_GREY      1       /* Linear greyscale */
 #define VIDEO_PALETTE_HI240     2       /* High 240 cube (BT848) */
@@ -98,6 +98,11 @@
 #define VIDEO_PALETTE_COMPONENT 7       /* start of component entries */
 #endif
 
+
+/* Debug levels */
+#define CAMERA_WARNINGS         3   /* warnings only */
+#define CAMERA_INFO             5   /* debug level to activate everything */
+
 /* Default picture settings */
 #define DEF_WIDTH              352
 #define DEF_HEIGHT             288
@@ -123,6 +128,9 @@
                                      * and then we show a grey image instead
                                      */
 
+#define CONNECTION_KO "Lost connection"
+#define CONNECTION_OK "Connection OK"
+
 #define DEF_MAXSTREAMS          10  /* Maximum number of webcam clients per camera */
 #define DEF_MAXWEBQUEUE         10  /* Maximum number of webcam client in queue */
 
@@ -248,6 +256,7 @@ struct context {
 	char conf_filename[PATH_MAX];
 	int threadnr;
 	int daemon;
+	char pid_file[PATH_MAX];
 
 	struct config conf;
 	struct images imgs;
@@ -274,7 +283,8 @@ struct context {
 
 	int event_nr;
 	int prev_event;
-	char text_event_string[255]; /* The text for conv. spec. %C - we assume 255 characters is fine */
+	char text_event_string[PATH_MAX]; /* The text for conv. spec. %C -
+	                                     we assume PATH_MAX normally 4096 characters is fine */
 
 	int shots;
 	struct tm *currenttime_tm;
@@ -291,8 +301,9 @@ struct context {
 	int switched;
 	int pause;
 	int missing_frame_counter;   /* counts failed attempts to fetch picture frame from camera */
+	int lost_connection;	
 
-#ifdef __freebsd__
+#if (defined(BSD))
 	int tuner_dev;
 #endif
 	int video_dev;
