@@ -52,9 +52,9 @@
 
 #define CONNECT_TIMEOUT        10   /* timeout on remote connection attempt */
 #define READ_TIMEOUT            5   /* default timeout on recv requests */
+#define POLLING_TIMEOUT  READ_TIMEOUT /* file polling timeout [s] */
+#define POLLING_TIME  500*1000*1000 /* file polling time quantum [ns] (500ms) */
 #define MAX_HEADER_RETRIES      5   /* Max tries to find a header record */
-#define CAMERA_WARNINGS         3   /* if debug_level >= this number print */
-#define CAMERA_INFO             5   /* debug level to activate everything */
 #define MINVAL(x, y) ((x) < (y) ? (x) : (y))
 
 /*
@@ -65,6 +65,10 @@
  */
 #define SETUP    ((cnt->conf.setup_mode) || (debug_level >= CAMERA_INFO))
 
+
+tfile_context *file_new_context(void);
+void file_free_context(tfile_context* ctxt);
+
 /* These strings are used for the HTTP connection */
 static const char    *connect_req = "GET %s HTTP/1.0\r\n"
                       "Host: %s\r\n"
@@ -151,6 +155,13 @@ static void netcam_url_parse(struct url_t *parse_url, const char *text_url)
 	regex_t pattbuf;
 	regmatch_t matches[10];
 
+	if( !strncmp( text_url, "file", 4 ) ) 
+		re = "(file)://(((.*):(.*))@)?"
+	                 "([^/:]|[-.a-z0-9]*)(:([0-9]*))?($|(/[^:][/-_.a-z0-9]+))";
+
+	if (debug_level > 7)
+		motion_log(-1, 0, "Entry netcam_url_parse data %s", text_url );
+
 	memset(parse_url, 0, sizeof(struct url_t));
 	/*
 	 * regcomp compiles regular expressions into a form that is
@@ -163,6 +174,8 @@ static void netcam_url_parse(struct url_t *parse_url, const char *text_url)
 		if (regexec(&pattbuf, text_url, 10, matches, 0) != REG_NOMATCH) {
 			for (i = 0; i < 10; i++) {
 				if ((s = netcam_url_match(matches[i], text_url)) != NULL) {
+					if (debug_level > 7)
+						motion_log(-1, 0, "Parse case %d data %s", i, s );
 					switch (i) {
 						case 1:
 							parse_url->service = s;
@@ -189,7 +202,7 @@ static void netcam_url_parse(struct url_t *parse_url, const char *text_url)
 			}
 		}
 	}
-	if (!parse_url->port) {
+	if ((!parse_url->port) && (parse_url->service)){
 		if (!strcmp(parse_url->service, "http"))
 			parse_url->port = 80;
 		else if (!strcmp(parse_url->service, "ftp"))
@@ -595,7 +608,6 @@ static void netcam_disconnect(netcam_context_ptr netcam)
 			motion_log(LOG_ERR, 1, "netcam_disconnect");
 
 		netcam->sock = -1;
-		rbuf_discard(netcam);
 	}
 }
 
@@ -606,12 +618,15 @@ static void netcam_disconnect(netcam_context_ptr netcam)
  *
  * Parameters:
  *
- *      netcam  pointer to netcam_context structure
+ *      netcam    pointer to netcam_context structure
+ *      err_flag  flag to suppress error printout (1 => suppress)
+ *                Note that errors which indicate something other than
+ *                a network connection problem are not suppressed.
  *
  * Returns:     0 for success, -1 for error
  *
  */
-static int netcam_connect(netcam_context_ptr netcam, struct timeval *timeout)
+static int netcam_connect(netcam_context_ptr netcam, int err_flag)
 {
 	struct sockaddr_in server;      /* for connect */
 	struct addrinfo *res;           /* for getaddrinfo */
@@ -633,8 +648,9 @@ static int netcam_connect(netcam_context_ptr netcam, struct timeval *timeout)
 
 	/* lookup the hostname given in the netcam URL */
 	if ((ret = getaddrinfo(netcam->connect_host, NULL, NULL, &res)) != 0) {
-		motion_log(LOG_ERR, 0, "getaddrinfo() failed (%s): %s",
-		           netcam->connect_host, gai_strerror(ret));
+		if (!err_flag)
+			motion_log(LOG_ERR, 0, "getaddrinfo() failed (%s): %s",
+			           netcam->connect_host, gai_strerror(ret));
 		netcam_disconnect(netcam);
 		return -1;
 	}
@@ -650,35 +666,21 @@ static int netcam_connect(netcam_context_ptr netcam, struct timeval *timeout)
 	server.sin_family = AF_INET;
 	server.sin_port = htons(netcam->connect_port);
 
-	if (timeout)
-		netcam->timeout = *timeout;
-
-	if ((setsockopt(netcam->sock, SOL_SOCKET, SO_RCVTIMEO,
-	                &netcam->timeout, sizeof(netcam->timeout))) < 0) {
-		motion_log(LOG_ERR, 1, "setsockopt() failed");
-		netcam_disconnect(netcam);
-		return -1;
-	}
-
 	/*
-	 * Unfortunately, the SO_RCVTIMEO does not affect the 'connect'
-	 * call, so we need to be quite a bit more clever.  We want to
-	 * use netcam->timeout, so we set the socket non-blocking and
-	 * then use a 'select' system call to control the timeout.
+	 * We set the socket non-blocking and then use a 'select'
+	 * system call to control the timeout.
 	 */
 
 	if ((saveflags = fcntl(netcam->sock, F_GETFL, 0)) < 0) {
 		motion_log(LOG_ERR, 1, "fcntl(1) on socket");
-		close(netcam->sock);
-		netcam->sock = -1;
+		netcam_disconnect(netcam);
 		return -1;
 	}
 
 	/* Set the socket non-blocking */
 	if (fcntl(netcam->sock, F_SETFL, saveflags | O_NONBLOCK) < 0) {
 		motion_log(LOG_ERR, 1, "fcntl(2) on socket");
-		close(netcam->sock);
-		netcam->sock = -1;
+		netcam_disconnect(netcam);
 		return -1;
 	}
 
@@ -687,19 +689,11 @@ static int netcam_connect(netcam_context_ptr netcam, struct timeval *timeout)
 	              sizeof(server));
 	back_err = errno;           /* save the errno from connect */
 
-	/* Restore the normal socket flags */
-	if (fcntl(netcam->sock, F_SETFL, saveflags) < 0) {
-		motion_log(LOG_ERR, 1, "fcntl(3) on socket");
-		close(netcam->sock);
-		netcam->sock = -1;
-		return -1;
-	}
-
 	/* If the connect failed with anything except EINPROGRESS, error */
 	if ((ret < 0) && (back_err != EINPROGRESS)) {
-		motion_log(LOG_ERR, 1, "connect() failed (%d)", back_err);
-		close(netcam->sock);
-		netcam->sock = -1;
+		if (!err_flag)
+			motion_log(LOG_ERR, 1, "connect() failed (%d)", back_err);
+		netcam_disconnect(netcam);
 		return -1;
 	}
 
@@ -711,9 +705,9 @@ static int netcam_connect(netcam_context_ptr netcam, struct timeval *timeout)
 	ret = select(FD_SETSIZE, NULL, &fd_w, NULL, &selecttime);
 
 	if (ret == 0) {            /* 0 means timeout */
-		motion_log(LOG_ERR, 0, "timeout on connect()");
-		close(netcam->sock);
-		netcam->sock = -1;
+		if (!err_flag)
+			motion_log(LOG_ERR, 0, "timeout on connect()");
+		netcam_disconnect(netcam);
 		return -1;
 	}
 
@@ -725,14 +719,15 @@ static int netcam_connect(netcam_context_ptr netcam, struct timeval *timeout)
 	len = sizeof(ret);
 
 	if (getsockopt(netcam->sock, SOL_SOCKET, SO_ERROR, &ret, &len) < 0) {
-		motion_log(LOG_ERR, 1, "getsockopt after connect");
+		motion_log(LOG_ERR, 0, "getsockopt after connect");
 		netcam_disconnect(netcam);
 		return -1;
 	}
 
 	/* If the return code is anything except 0, error on connect */
 	if (ret) {
-		motion_log(LOG_ERR, 0, "connect returned error");
+		if (!err_flag)
+			motion_log(LOG_ERR, 1, "connect returned error");
 		netcam_disconnect(netcam);
 		return -1;
 	}
@@ -981,8 +976,7 @@ static int netcam_read_html_jpeg(netcam_context_ptr netcam)
 				retval = netcam_recv(netcam, netcam->response->buffer +
 				                     netcam->response->buffer_left,
 				                     sizeof(netcam->response->buffer) -
-				                     netcam->response->buffer_left,
-				                     NULL);
+				                     netcam->response->buffer_left);
 
 				if (retval <= 0) { /* this is a fatal error */
 					motion_log(LOG_ERR, 1, "recv() fail after boundary string");
@@ -1145,6 +1139,179 @@ static int netcam_read_ftp_jpeg(netcam_context_ptr netcam)
 	return 0;
 }
 
+
+/**
+ * netcam_read_file_jpeg
+ *
+ *      This routine reads local image file. ( netcam_url file:///path/image.jpg )
+ *      The current implementation is still a little experimental,
+ *      and needs some additional code for error detection and
+ *      recovery.
+ */
+static int netcam_read_file_jpeg(netcam_context_ptr netcam)
+{
+	int loop_counter=0;
+	if (debug_level > 9) {
+		motion_log(-1,0,"Begin %s", __FUNCTION__);
+	}
+	netcam_buff_ptr buffer;
+	int len;
+	netcam_buff *xchg;
+	struct timeval curtime;
+	struct stat statbuf;
+
+	/* Point to our working buffer */
+	buffer = netcam->receiving;
+	buffer->used = 0;
+
+	/*int fstat(int filedes, struct stat *buf);*/
+	do {
+
+		if( stat( netcam->file->path, &statbuf) ) {
+			motion_log(-1, 0, "stat(%s) error", netcam->file->path );
+			return -1;
+		}
+	
+		if (debug_level > 9) {
+			motion_log(-1, 0, "statbuf.st_mtime[%d] != last_st_mtime[%d]", statbuf.st_mtime,  netcam->file->last_st_mtime);
+		}
+
+		if( loop_counter>((POLLING_TIMEOUT*1000*1000)/(POLLING_TIME/1000)) ) { //its waits POLLING_TIMEOUT
+			motion_log(-1, 0, "waiting new file image timeout" );
+			return -1;
+		}
+
+		if (debug_level > 9) {
+			motion_log(-1, 0, "delay waiting new file image ");
+		}
+
+		//SLEEP(netcam->timeout.tv_sec, netcam->timeout.tv_usec*1000 ); //its waits 5seconds - READ_TIMEOUT
+		SLEEP( 0, POLLING_TIME ); // its waits 500ms
+		/*return -1;*/
+		loop_counter++;
+
+	} while(statbuf.st_mtime==netcam->file->last_st_mtime);
+
+	netcam->file->last_st_mtime = statbuf.st_mtime;
+	if (debug_level > 5) {
+		motion_log(LOG_INFO, 0, "processing new file image - st_mtime "
+                              "%d", netcam->file->last_st_mtime );
+	}
+
+	/* Assure there's enough room in the buffer */
+	while( buffer->size < (size_t)statbuf.st_size ) {
+		netcam_check_buffsize(buffer, statbuf.st_size ); 
+	}
+
+	/* Do the read */
+	netcam->file->control_file_desc = open( netcam->file->path, O_RDONLY );
+	if( netcam->file->control_file_desc < 0 ) {
+		motion_log(-1, 0, "open(%s) error:%d", netcam->file->path, netcam->file->control_file_desc );
+		return -1;
+	}
+
+	if ((len = read(netcam->file->control_file_desc, buffer->ptr + buffer->used, statbuf.st_size)) < 0) {
+		motion_log(-1, 0, "read(%s) error:%d", netcam->file->control_file_desc, len );
+		return -1;
+	}
+
+	buffer->used += len;
+
+	close( netcam->file->control_file_desc );
+
+	if (gettimeofday(&curtime, NULL) < 0) {
+		motion_log(LOG_ERR, 1, "gettimeofday in netcam_read_jpeg");
+	}
+
+	netcam->receiving->image_time = curtime;
+	/*
+	 * Calculate our "running average" time for this netcam's
+	 * frame transmissions (except for the first time).
+	 * Note that the average frame time is held in microseconds.
+	 */
+	if (netcam->last_image.tv_sec) {
+		netcam->av_frame_time =
+		  ((9.0 * netcam->av_frame_time) + 1000000.0 *
+		  (curtime.tv_sec - netcam->last_image.tv_sec) +
+		  (curtime.tv_usec- netcam->last_image.tv_usec))
+		  / 10.0;
+		if (debug_level > 5)
+			motion_log(-1, 0, "Calculated frame time %f", netcam->av_frame_time);
+	}
+
+	netcam->last_image = curtime;
+
+	/*
+	 * read is complete - set the current 'receiving' buffer atomically
+	 * as 'latest', and make the buffer previously in 'latest' become
+	 * the new 'receiving'
+	 */
+	pthread_mutex_lock(&netcam->mutex);
+
+	xchg = netcam->latest;
+	netcam->latest = netcam->receiving;
+	netcam->receiving = xchg;
+	netcam->imgcnt++;
+
+	/*
+	 * We have a new frame ready.  We send a signal so that
+	 * any thread (e.g. the motion main loop) waiting for the
+	 * next frame to become available may proceed.
+	 */
+	pthread_cond_signal(&netcam->pic_ready);
+
+	pthread_mutex_unlock(&netcam->mutex);
+
+	if (debug_level > 9) {
+		motion_log(-1,0,"End %s", __FUNCTION__);
+	}
+	return 0;
+}
+
+
+tfile_context *file_new_context(void) {
+	tfile_context *ret;
+
+	/* note that mymalloc will exit on any problem */
+	ret = mymalloc(sizeof(tfile_context));
+	if (!ret)
+		return ret;
+
+	memset(ret, 0, sizeof(tfile_context));
+	return ret;
+}
+
+void file_free_context(tfile_context* ctxt) {
+	if (ctxt == NULL)
+		return;
+
+	if (ctxt->path != NULL)
+		free(ctxt->path);
+
+	free(ctxt);
+}
+
+static int netcam_setup_file(netcam_context_ptr netcam, struct url_t *url) {
+
+	if ((netcam->file = file_new_context()) == NULL)
+		return -1;
+
+	/*
+	 * We copy the strings out of the url structure into the ftp_context
+	 * structure.  By setting url->{string} to NULL we effectively "take
+	 * ownership" of the string away from the URL (i.e. it won't be freed
+	 * when we cleanup the url structure later).
+	 */
+	netcam->file->path = url->path;
+	url->path = NULL;
+
+	netcam_url_free(url);
+
+	netcam->get_image = netcam_read_file_jpeg;
+
+	return 0;
+}
+
 /**
  * netcam_handler_loop
  *      This is the "main loop" for the handler thread.  It is created
@@ -1160,6 +1327,7 @@ static int netcam_read_ftp_jpeg(netcam_context_ptr netcam)
 static void *netcam_handler_loop(void *arg)
 {
 	int retval;
+	int open_error = 0;
 	netcam_context_ptr netcam = arg;
 	struct context *cnt = netcam->cnt; /* needed for the SETUP macro :-( */
 
@@ -1186,11 +1354,21 @@ static void *netcam_handler_loop(void *arg)
 		if (netcam->response) {    /* if html input */
 			if (!netcam->caps.streaming) {
 
-				if (netcam_connect(netcam, NULL) < 0) {
-					motion_log(LOG_ERR, 0, "re-opening camera (non-streaming)");
+				if (netcam_connect(netcam, open_error) < 0) {
+					if (!open_error) { /* log first error */
+						motion_log(LOG_ERR, 0,
+						    "re-opening camera (non-streaming)");
+						open_error = 1;
+					}
 					/* need to have a dynamic delay here */
+					SLEEP(5,0);
 					continue;
 				}
+				if (open_error) {          /* log re-connection */
+					motion_log(LOG_ERR, 0,
+					    "camera re-connected");
+					open_error = 0;
+				}
 				/* Send our request and look at the response */
 				if ((retval = netcam_read_first_header(netcam)) != 1) {
 					if (retval > 0) {
@@ -1205,9 +1383,12 @@ static void *netcam_handler_loop(void *arg)
 				}
 			} else {
 				if (netcam_read_next_header(netcam) < 0) {
-					if (netcam_connect(netcam, NULL) < 0) {
-						motion_log(LOG_ERR, 0,
-						    "re-opening camera (streaming)");
+					if (netcam_connect(netcam, open_error) < 0) {
+						if (!open_error) { /* log first error */
+							motion_log(LOG_ERR, 0,
+							    "re-opening camera (streaming)");
+							open_error = 1;
+						}
 						SLEEP(5,0);
 						continue;
 					}
@@ -1225,6 +1406,11 @@ static void *netcam_handler_loop(void *arg)
 						continue;
 					}
 				}
+				if (open_error) {          /* log re-connection */
+					motion_log(LOG_ERR, 0,
+					    "camera re-connected");
+					open_error = 0;
+				}
 			}
 		}
 		if (netcam->get_image(netcam) < 0) {
@@ -1420,7 +1606,7 @@ static int netcam_setup_html(netcam_context_ptr netcam, struct url_t *url) {
 		 * netcam_connect does an automatic netcam_close, so it's
 		 * safe to include it as part of this loop
 		 */
-		if (netcam_connect(netcam, NULL) != 0) {
+		if (netcam_connect(netcam, 0) != 0) {
 			motion_log(LOG_ERR, 0,"Failed to open camera - check your config and that netcamera is online");
 
 			/* Fatal error on startup */
@@ -1520,33 +1706,27 @@ static int netcam_setup_ftp(netcam_context_ptr netcam, struct url_t *url) {
  *      netcam          Pointer to a netcam context
  *      buffptr         Pointer to the receive buffer
  *      buffsize        Length of the buffer
- *      timeout         Pointer to struct timeval (NULL for no change)
  *
  * Returns:
  *      If successful, the length of the message received, otherwise the
  *      error reply from the system call.
  *
  */
-ssize_t netcam_recv(netcam_context_ptr netcam, void *buffptr, size_t buffsize,
-                    struct timeval *timeout) {
+ssize_t netcam_recv(netcam_context_ptr netcam, void *buffptr, size_t buffsize) {
 	ssize_t retval;
+	fd_set fd_r;
+	struct timeval selecttime;
 
-	if (timeout) {
-		if ((timeout->tv_sec != netcam->timeout.tv_sec) ||
-		    (timeout->tv_usec != netcam->timeout.tv_usec)) {
-			if ((setsockopt(netcam->sock, SOL_SOCKET,
-			     SO_RCVTIMEO, timeout, sizeof(*timeout))) < 0) {
-				motion_log(LOG_ERR, 1, "setsockopt() in netcam_recv failed");
-			}
-		}
-		netcam->timeout = *timeout;
+	FD_ZERO(&fd_r);
+	FD_SET(netcam->sock, &fd_r);
+	selecttime = netcam->timeout;
+
+	retval = select(FD_SETSIZE, &fd_r, NULL, NULL, &selecttime);
+	if (retval == 0) {             /* 0 means timeout */
+		return -1;
 	}
 
-	do {
-		retval = recv(netcam->sock, buffptr, buffsize, 0);
-	} while ((retval < 0) && ((errno == EINTR) || (errno == EAGAIN)));
-
-	return retval;
+	return recv(netcam->sock, buffptr, buffsize, 0);
 }
 
 /**
@@ -1684,12 +1864,14 @@ void netcam_cleanup(netcam_context_ptr netcam, int init_retry_flag)
 		free(netcam->jpegbuf);
 	}
 
-	if (netcam->response != NULL) {
-		free(netcam->response);
-	}
-
 	if (netcam->ftp != NULL) {
 		ftp_free_context(netcam->ftp);
+	} else {
+		netcam_disconnect(netcam);
+	}
+
+	if (netcam->response != NULL) {
+		free(netcam->response);
 	}
 
 	pthread_mutex_destroy(&netcam->mutex);
@@ -1872,14 +2054,15 @@ int netcam_start(struct context *cnt)
 		netcam->connect_port = url.port;
 	}
 
-
-	if (!strcmp(url.service, "http")) {
+	if ((url.service) && (!strcmp(url.service, "http")) ){
 		retval = netcam_setup_html(netcam, &url);
-	} else if (!strcmp(url.service, "ftp")) {
+	} else if ((url.service) && (!strcmp(url.service, "ftp")) ){
 		retval = netcam_setup_ftp(netcam, &url);
+	} else if ((url.service) && (!strcmp(url.service, "file")) ){
+		retval = netcam_setup_file(netcam, &url);
 	} else {
 		motion_log(LOG_ERR, 0, "Invalid netcam service  '%s' - "
-		                       "must be http or ftp", url.service);
+					"must be http or ftp", url.service);
 		netcam_url_free(&url);
 		return -1;
 	}
@@ -1895,7 +2078,7 @@ int netcam_start(struct context *cnt)
 	 * these to set the required image buffer(s) in our netcam_struct.
 	 */
 	if ((retval = netcam->get_image(netcam)) != 0) {
-		motion_log(LOG_ERR, 0, "Failed trying to read first image");
+		motion_log(LOG_ERR, 0, "Failed trying to read first image - retval:%d", retval );
 		return -1;
 	}
 
