@@ -274,7 +274,7 @@ void rbuf_initialize(netcam_context_ptr netcam)
 int rbuf_read_bufferful(netcam_context_ptr netcam)
 {
 	return netcam_recv(netcam, netcam->response->buffer,
-	                   sizeof (netcam->response->buffer), 0);
+	                   sizeof (netcam->response->buffer));
 }
 
 /* Like rbuf_readchar(), only don't move the buffer position.  */
@@ -282,10 +282,9 @@ int rbuf_peek(netcam_context_ptr netcam, char *store)
 {
 	if (!netcam->response->buffer_left) {
 		int res;
-		netcam->response->buffer_pos = netcam->response->buffer;
-		netcam->response->buffer_left = 0;
+		rbuf_initialize(netcam);
 		res = netcam_recv (netcam, netcam->response->buffer,
-		                   sizeof (netcam->response->buffer), NULL);
+		                   sizeof (netcam->response->buffer));
 
 		if (res <= 0)
 			return res;
@@ -318,13 +317,6 @@ int rbuf_flush(netcam_context_ptr netcam, char *where, int maxsize)
 	}
 }
 
-/* Discard any cached data in RBUF.  */
-void rbuf_discard (netcam_context_ptr netcam)
-{
-	netcam->response->buffer_left = 0;
-	netcam->response->buffer_pos = netcam->response->buffer;
-}
-
 /* Get the HTTP result code */
 int http_result_code(const char *header)
 {
