@@ -14,12 +14,12 @@
 
 void overlay_smartmask(struct context *, unsigned char *);
 void overlay_fixed_mask(struct context *, unsigned char *);
-void put_fixed_mask(struct context *, char *);
+void put_fixed_mask(struct context *, const char *);
 void overlay_largest_label(struct context *, unsigned char *);
-void put_picture_fd (struct context *, FILE *, unsigned char *, int);
-int put_picture_memory (struct context *, unsigned char*, int, unsigned char *, int);
-void put_picture (struct context *, char *, unsigned char *, int );
-unsigned char *get_pgm (struct context *, FILE *, int, int);
+void put_picture_fd(struct context *, FILE *, unsigned char *, int);
+int put_picture_memory(struct context *, unsigned char*, int, unsigned char *, int);
+void put_picture(struct context *, char *, unsigned char *, int );
+unsigned char *get_pgm(FILE *, int, int);
 void preview_best(struct context *);
 
 #endif /* _INCLUDE_PICTURE_H_ */
