@@ -14,6 +14,8 @@
  *	increases the Motion CPU usage slightly.
  *
  *	Version history:
+ *	  v6 (29-Aug-2005) - simplified the code as Motion now requires
+ *	                     that width and height are multiples of 16
  *	  v5 (3-Aug-2005)  - cleanup in code comments
  *	                   - better adherence to coding standard
  *	                   - fix for __bswap_32 macro collision
@@ -48,7 +50,7 @@ typedef unsigned TYPE_32BIT __uint32;
 /**
  * The code below is copied (with modification) from bits/byteswap.h. It provides
  * a macro/function named rot__bswap_32 that swaps the bytes in a 32-bit integer, 
- * preferrably using the bswap assembler instruction if configure found support
+ * preferably using the bswap assembler instruction if configure found support
  * for it.
  *
  * It would be neater to simply include byteswap.h and use the bswap_32 macro
@@ -132,7 +134,7 @@ static inline __uint32 rot__bswap_32(__uint32 __bsx)
  *
  * Returns: nothing
  */
-void reverse_inplace_quad(unsigned char *src, int size) 
+static void reverse_inplace_quad(unsigned char *src, int size) 
 {
 	__uint32 *nsrc = (__uint32 *)src;              /* first quad */
 	__uint32 *ndst = (__uint32 *)(src + size - 4); /* last quad */
@@ -145,33 +147,6 @@ void reverse_inplace_quad(unsigned char *src, int size)
 	}
 }
 
-/**
- * reverse_inplace_single
- * 
- *  Reverses a block of memory in-place, 1 byte at a time. This function
- *  is slower than reverse_inplace_quad, and should only be used when the
- *  size of the memory block isn't divisible by 4 (which seldom happens,
- *  I guess).
- *
- * Parameters:
- * 
- *   src  - the memory block to reverse
- *   size - the size (in bytes) of the memory block
- *
- * Returns: nothing
- */
-void reverse_inplace_single(unsigned char *src, int size) 
-{
-	register unsigned char tmp;
-	unsigned char *dst = src + size - 1; /* last byte */
-	
-	while (src < dst) {
-		tmp = *dst;
-		*dst-- = *src;
-		*src++ = tmp;
-	}
-}
-
 /**
  * rot90cw
  * 
@@ -189,8 +164,8 @@ void reverse_inplace_single(unsigned char *src, int size)
  *
  * Returns: nothing
  */
-void rot90cw(unsigned char *src, register unsigned char *dst, int size,
-             int width, int height) 
+static void rot90cw(unsigned char *src, register unsigned char *dst, int size,
+                    int width, int height) 
 {
 	unsigned char *endp;
 	register unsigned char *base;
@@ -222,8 +197,8 @@ void rot90cw(unsigned char *src, register unsigned char *dst, int size,
  *
  * Returns: nothing
  */
-inline void rot90ccw(unsigned char *src, register unsigned char *dst, int size, 
-                     int width, int height) 
+static inline void rot90ccw(unsigned char *src, register unsigned char *dst,
+                            int size, int width, int height) 
 {
 	unsigned char *endp;
 	register unsigned char *base;
@@ -253,7 +228,7 @@ inline void rot90ccw(unsigned char *src, register unsigned char *dst, int size,
  */
 void rotate_init(struct context *cnt) 
 {
-	int size, multiple;
+	int size;
 	
 	/* Make sure temp_buf isn't freed if it hasn't been allocated. */
 	cnt->rotate_data.temp_buf = NULL;
@@ -262,7 +237,7 @@ void rotate_init(struct context *cnt)
 	 * we have a value that is safe from changes caused by motion-control.
 	 */
 	if((cnt->conf.rotate_deg % 90) > 0) {
-		motion_log(cnt, LOG_ERR, 0, "Config option \"rotate\" not a multiple of 90: %d",
+		motion_log(LOG_ERR, 0, "Config option \"rotate\" not a multiple of 90: %d",
 		           cnt->conf.rotate_deg);
 		cnt->conf.rotate_deg = 0;     /* disable rotation */
 		cnt->rotate_data.degrees = 0; /* force return below */
@@ -301,42 +276,28 @@ void rotate_init(struct context *cnt)
 	{
 	case VIDEO_PALETTE_YUV420P:
 		/* For YUV 4:2:0 planar, the memory block used for 90/270 degrees
-		 * rotation needs to be width x height x 1.5 bytes large. Also, 
-		 * width x height needs to be divisible by 16 for reverse_inplace_quad 
-		 * to be used (because the U and V planes must be divisible by 4, and 
-		 * they are each four times smaller than the Y plane, which is width x
-		 * height bytes in size).
+		 * rotation needs to be width x height x 1.5 bytes large. 
 		 */
 		size = cnt->imgs.width * cnt->imgs.height * 3 / 2;
-		multiple = 16;
 		break;
 	case VIDEO_PALETTE_GREY:
 		/* For greyscale, the memory block used for 90/270 degrees rotation
-		 * needs to be width x height bytes large. Also, width x height needs
-		 * to be divisible by 4 for reverse_inplace_quad to be used.
+		 * needs to be width x height bytes large.
 		 */
 		size = cnt->imgs.width * cnt->imgs.height;
-		multiple = 4;
 		break;
 	default:
 		cnt->rotate_data.degrees = 0;
-		motion_log(cnt, LOG_ERR, 0, "Unsupported palette (%d), rotation is disabled",
+		motion_log(LOG_ERR, 0, "Unsupported palette (%d), rotation is disabled",
 		           cnt->imgs.type);
 		return;
 	}
 
-	/* Set the rot180 pointer to point to the appropriate reverse function. */
-	if((cnt->imgs.width * cnt->imgs.height) % multiple > 0) {
-		cnt->rotate_data.rotate_180 = &reverse_inplace_single;
-	} else {
-		cnt->rotate_data.rotate_180 = &reverse_inplace_quad;
-	}
-
 	/* Allocate memory if rotating 90 or 270 degrees, because those rotations 
 	 * cannot be performed in-place (they can, but it would be too slow).
 	 */
 	if((cnt->rotate_data.degrees == 90) || (cnt->rotate_data.degrees == 270)) {
-		cnt->rotate_data.temp_buf = mymalloc(cnt, size);
+		cnt->rotate_data.temp_buf = mymalloc(size);
 	}
 }
 
@@ -430,10 +391,10 @@ int rotate_map(struct context *cnt, unsigned char *map)
 		/* 180 degrees is easy - just reverse the data within
 		 * Y, U and V.
 		 */
-		(*cnt->rotate_data.rotate_180)(map, wh);
+		reverse_inplace_quad(map, wh);
 		if(cnt->imgs.type == VIDEO_PALETTE_YUV420P) {
-			(*cnt->rotate_data.rotate_180)(map + wh, wh4);
-			(*cnt->rotate_data.rotate_180)(map + wh + wh4, wh4);
+			reverse_inplace_quad(map + wh, wh4);
+			reverse_inplace_quad(map + wh + wh4, wh4);
 		}
 		break;
 
