@@ -17,6 +17,9 @@
  *	along with this program; if not, write to the Free Software
  *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
+
+#include "picture.h"
+/* picture.h already includes motion.h and this one webcam.h */
 #include <sys/time.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -25,24 +28,22 @@
 #include <sys/fcntl.h>
 #include <sys/stat.h>
 
-#include "picture.h"
-#include "webcam.h"
-#include "motion.h"
 
 
-/* This function sets up a TCP/IP socket for incomming requests. It is called only during
- * initiatization of Motion from the function webcam_init
+
+/* This function sets up a TCP/IP socket for incoming requests. It is called only during
+ * initialisation of Motion from the function webcam_init
  * The function sets up a a socket on the port number given by _port_.
  * If the parameter _local_ is not zero the socket is setup to only accept connects from localhost.
  * Otherwise any client IP address is accepted. The function returns an integer representing the socket.
  */
-int http_bindsock(struct context *cnt, int port, int local)
+int http_bindsock(int port, int local)
 {
 	int sl, optval=1;
 	struct sockaddr_in sin;
 
 	if ((sl=socket(PF_INET, SOCK_STREAM, 0))<0) {
-		motion_log(cnt, LOG_ERR, 1, "socket()");
+		motion_log(LOG_ERR, 1, "socket()");
 		return -1;
 	}
 
@@ -58,13 +59,13 @@ int http_bindsock(struct context *cnt, int port, int local)
 	setsockopt(sl, SOL_SOCKET, SO_REUSEADDR, &optval, sizeof(optval));
 
 	if (bind(sl, (struct sockaddr *)&sin, sizeof(struct sockaddr_in))==-1) {
-		motion_log(cnt, LOG_ERR, 1, "bind()");
+		motion_log(LOG_ERR, 1, "bind()");
 		close(sl);
 		return -1;
 	}
 
 	if (listen(sl, DEF_MAXWEBQUEUE)==-1) {
-		motion_log(cnt, LOG_ERR, 1, "listen()");
+		motion_log(LOG_ERR, 1, "listen()");
 		close(sl);
 		return -1;
 	}
@@ -73,7 +74,7 @@ int http_bindsock(struct context *cnt, int port, int local)
 }
 
 
-int http_acceptsock(struct context *cnt, int sl)
+static int http_acceptsock(int sl)
 {
 	int sc;
 	unsigned long i;
@@ -86,7 +87,7 @@ int http_acceptsock(struct context *cnt, int sl)
 		return sc;
 	}
 	
-	motion_log(cnt, LOG_ERR, 1, "accept()");
+	motion_log(LOG_ERR, 1, "accept()");
 
 	return -1;
 }
@@ -97,7 +98,7 @@ int http_acceptsock(struct context *cnt, int sl)
  * to be sent (either because there isn't any, or because the clients
  * are not able to accept it).
  */
-void webcam_flush(struct webcam *list, int *stream_count, int lim)
+static void webcam_flush(struct webcam *list, int *stream_count, int lim)
 {
 	int written;            /* the number of bytes actually written */
 	struct webcam *client;  /* pointer to the client being served */
@@ -131,7 +132,7 @@ void webcam_flush(struct webcam *list, int *stream_count, int lim)
 					      client->tmpbuffer->size - client->filepos);
 		
 				/* If any data has been written, update the
-				 * data pointer and set the "workdone flag
+				 * data pointer and set the workdone flag
 				 */
 				if (written>0) {
 					client->filepos+=written;
@@ -147,7 +148,6 @@ void webcam_flush(struct webcam *list, int *stream_count, int lim)
 			 */
 			if ( (client->filepos >= client->tmpbuffer->size) ||
 			     (written < 0 && errno!=EAGAIN)) {
-				
 				/* If no other clients need this buffer, free it */
 				if (--client->tmpbuffer->ref <= 0) {
 					free(client->tmpbuffer->ptr);
@@ -197,24 +197,19 @@ void webcam_flush(struct webcam *list, int *stream_count, int lim)
 /* Routine to create a new "tmpbuffer", which is a common
  * object used by all clients connected to a single camera
  */
-struct webcam_buffer *webcam_tmpbuffer(struct context *cnt, int size)
+static struct webcam_buffer *webcam_tmpbuffer(int size)
 {
-	struct webcam_buffer *tmpbuffer=mymalloc(cnt, sizeof(struct webcam_buffer));
+	struct webcam_buffer *tmpbuffer=mymalloc(sizeof(struct webcam_buffer));
 	tmpbuffer->ref=0;
-	tmpbuffer->ptr=mymalloc(cnt, size);
-	
-	if (!tmpbuffer->ptr) {
-		free(tmpbuffer);
-		return NULL;
-	}
-	
+	tmpbuffer->ptr=mymalloc(size);
+		
 	return tmpbuffer;
 }
 
 
-void webcam_add_client(struct context *cnt, struct webcam *list, int sc)
+static void webcam_add_client(struct webcam *list, int sc)
 {
-	struct webcam *new=mymalloc(cnt, sizeof(struct webcam));
+	struct webcam *new = mymalloc(sizeof(struct webcam));
 	static const char header[] = "HTTP/1.0 200 OK\r\n"
 			"Server: Motion/"VERSION"\r\n"
 			"Connection: close\r\n"
@@ -223,18 +218,17 @@ void webcam_add_client(struct context *cnt, struct webcam *list, int sc)
 			"Cache-Control: no-cache, private\r\n"
 			"Pragma: no-cache\r\n"
 			"Content-Type: multipart/x-mixed-replace; boundary=--BoundaryString\r\n\r\n";
-			
+
+	memset(new, 0, sizeof(struct webcam));
 	new->socket=sc;
 	
-	if ((new->tmpbuffer = webcam_tmpbuffer(cnt, sizeof(header))) == NULL) {
-		motion_log(cnt, LOG_ERR, 1, "Error creating tmpbuffer in webcam_add_client");
+	if ((new->tmpbuffer = webcam_tmpbuffer(sizeof(header))) == NULL) {
+		motion_log(LOG_ERR, 1, "Error creating tmpbuffer in webcam_add_client");
 	} else {
 		memcpy(new->tmpbuffer->ptr, header, sizeof(header)-1);
 		new->tmpbuffer->size = sizeof(header)-1;
 	}
 	
-	new->filepos=0;
-	new->nr=0;
 	new->prev=list;
 	new->next=list->next;
 	
@@ -242,11 +236,10 @@ void webcam_add_client(struct context *cnt, struct webcam *list, int sc)
 		new->next->prev=new;
 	
 	list->next=new;
-	new->last=0;
 }
 
 
-void webcam_add_write(struct webcam *list, struct webcam_buffer *tmpbuffer, int fps)
+static void webcam_add_write(struct webcam *list, struct webcam_buffer *tmpbuffer, unsigned int fps)
 {
 	struct timeval curtimeval;
 	unsigned long int curtime;
@@ -257,7 +250,7 @@ void webcam_add_write(struct webcam *list, struct webcam_buffer *tmpbuffer, int
 	while (list->next) {
 		list=list->next;
 		
-		if (list->tmpbuffer==NULL && (curtime-list->last)>=1000000L/fps) {
+		if (list->tmpbuffer==NULL && ((curtime-list->last) >= 1000000L/fps)) {
 			list->last=curtime;
 			list->tmpbuffer=tmpbuffer;
 			tmpbuffer->ref++;
@@ -277,7 +270,7 @@ void webcam_add_write(struct webcam *list, struct webcam_buffer *tmpbuffer, int
  * We return 1 if it finds a list->tmpbuffer which is a NULL pointer which would
  * be the next client ready to be sent a new image. If not a 0 is returned.
  */
-int webcam_check_write(struct webcam *list)
+static int webcam_check_write(struct webcam *list)
 {
 	while (list->next) {
 		list=list->next;
@@ -290,12 +283,12 @@ int webcam_check_write(struct webcam *list)
 
 
 /* This function is called from motion.c for each motion thread starting up.
- * The function setup the incomming tcp socket that the clients connect to
+ * The function setup the incoming tcp socket that the clients connect to
  * The function returns an integer representing the socket.
  */
 int webcam_init(struct context *cnt)
 {
-	cnt->webcam.socket=http_bindsock(cnt, cnt->conf.webcam_port, cnt->conf.webcam_localhost);
+	cnt->webcam.socket=http_bindsock(cnt->conf.webcam_port, cnt->conf.webcam_localhost);
 	cnt->webcam.next=NULL;
 	cnt->webcam.prev=NULL;
 	return cnt->webcam.socket;
@@ -310,12 +303,12 @@ void webcam_stop(struct context *cnt)
 	struct webcam *next = cnt->webcam.next;
 
 	if (cnt->conf.setup_mode)
-		motion_log(cnt, -1, 0, "Closing webcam listen socket");
+		motion_log(-1, 0, "Closing webcam listen socket");
 	
 	close(cnt->webcam.socket);
 	
 	if (cnt->conf.setup_mode)
-		motion_log(cnt, LOG_ERR, 1, "Closing active webcam sockets");
+		motion_log(LOG_ERR, 1, "Closing active webcam sockets");
 
 	while (next) {
 		list=next;
@@ -375,8 +368,8 @@ void webcam_put(struct context *cnt, unsigned char *image)
 	 */
 	if ((cnt->stream_count < DEF_MAXSTREAMS) &&
 	    (select(sl+1, &fdread, NULL, NULL, &timeout)>0)) {
-		sc=http_acceptsock(cnt, sl);
-		webcam_add_client(cnt, &cnt->webcam, sc);
+		sc=http_acceptsock(sl);
+		webcam_add_client(&cnt->webcam, sc);
 		cnt->stream_count++;
 	}
 	
@@ -390,7 +383,7 @@ void webcam_put(struct context *cnt, unsigned char *image)
 		 * than necessary, but it is difficult to estimate the
 		 * minimum size actually required.
 		 */
-		tmpbuffer=webcam_tmpbuffer(cnt, cnt->imgs.size);
+		tmpbuffer = webcam_tmpbuffer(cnt->imgs.size);
 		
 		/* check if allocation went ok */
 		if (tmpbuffer) {
@@ -434,7 +427,7 @@ void webcam_put(struct context *cnt, unsigned char *image)
 			 */
 			webcam_add_write(&cnt->webcam, tmpbuffer, cnt->conf.webcam_maxrate);
 		} else {
-			motion_log(cnt, LOG_ERR, 1, "Error creating tmpbuffer");
+			motion_log(LOG_ERR, 1, "Error creating tmpbuffer");
 		}
 	}
 	
