@@ -21,8 +21,8 @@
 #ifndef _INCLUDE_WEBCAM_H_
 #define _INCLUDE_WEBCAM_H_
 
-struct webcam_tmpfile {
-	FILE *fd;
+struct webcam_buffer {
+	unsigned char *ptr;
 	int ref;
 	long size;
 };
@@ -30,7 +30,7 @@ struct webcam_tmpfile {
 struct webcam {
 	int socket;
 	FILE *fwrite;
-	struct webcam_tmpfile *tmpfile;
+	struct webcam_buffer *tmpbuffer;
 	long filepos;
 	int nr;
 	unsigned long int last;
@@ -39,7 +39,7 @@ struct webcam {
 };
 
 int webcam_init(struct context *);
-int webcam_put(struct context *, char *);
+void webcam_put(struct context *, unsigned char *);
 void webcam_stop(struct context *);
 
 #endif /* _INCLUDE_WEBCAM_H_ */
